<?php

/**
 * @file
 *
 * FileLogger
 */

namespace Pyramid\Component\Logger;

use Exception;

class FileLogger {
    
    /*
     * 文件句柄
     * @var resource
     */
    protected $handler;
    
    /*
     * 文件
     * @var string
     */
    protected $logFile;
    
    /*
     * 等级
     * @var string
     */
    protected $level = 0;
    
    /*
     * 是否可写入
     * @var bool
     */
    protected $writeAble = false;

    /**
     * 析构函数
     */
    public function __construct($config = array()) {
        if (isset($config['file'])) {
            $this->logFile = $config['file'];
        } else {
            $this->logFile = 'log';
        }
        $this->logFile .= '.' . date('Ymd') . '.log';
        $this->handler = fopen($this->logFile, 'ab');
        if (is_resource($this->handler)) {
            $this->writeAble = true;
        }
        if (isset($config['level'])) {
            $this->level = (int) $config['level'];
        }
    }

    /**
     * 致命错误
     */
    public function fatal($string) {
        if ($this->level & Logger::FATAL) {
            $this->write($string, 'FATAL');
        }
    }
    
    /**
     * 一般错误
     */
    public function error($string) {
        if ($this->level & Logger::ERROR) {
            $this->write($string, 'ERROR');
        }
    }
    
    /**
     * 应用警告
     */
    public function warn($string) {
        if ($this->level & Logger::WARN) {
            $this->write($string, 'WARN');
        }
    }
    
    /**
     * 基本信息
     */
    public function info($string) {
        if ($this->level & Logger::INFO) {
            $this->write($string, 'INFO');
        }
    }
    
    /**
     * Debug
     */
    public function debug($string) {
        if ($this->level & Logger::DEBUG) {
            $this->write($string, 'DEBUG');
        }
    }
    
    /**
     * Trace
     */
    public function trace($string) {
        if ($this->level & Logger::TRACE) {
            $this->write($string, 'TRACE');
        }
    }

    /**
     * Log
     */
    public function log($string) {
        $this->write($string, 'LOG');
    }

    /**
     * 析构函数
     */
    public function __destruct() {
        $this->handler && fclose($this->handler);
    }

    //执行写日志
    protected function write($string, $levelName = '') {
        if (!$this->writeAble) {
            return false;
        }
        flock($this->handler, LOCK_EX);
        fwrite($this->handler, sprintf("%s [%s] %s\n", date('c'), $levelName, $string));
        flock($this->handler, LOCK_UN);
    }

}
