<?php

/**
 * @file
 *  admin config of the xwechat module.
 *
 */

use Pyramid\Component\WeChat\WeChat;
use Pyramid\Component\WeChat\Request;
use Pyramid\Component\WeChat\Response;

/**
 * Form with overview of all xwechat config.
 */
function xwechat_config_add_form($form, &$form_state) {
  static $bundles = array('wechat' => '公众号', 'wechatcorp' => '企业号');
  // get xwechat config.
  $rows = array();
  $configs = xwechat_get_configs(NULL);

  foreach ($configs as $config) {
    $links = module_invoke_all('xwechat_operation', array(), $config);
    $rows[] = array(
      'wid' => $config->wid,
      'qrcode' => $config->username ? "<img style='max-width:100px' src='http://open.weixin.qq.com/qr/code/?username={$config->username}' />" : '',
      'name' => check_plain($config->name),
      'config' => 'AppID: ' . $config->appid . ' </strong>'
                  .'<br>Token: ' . xwechat_config_mask($config->token)
                  .'<br>AppSecret: ' . xwechat_config_mask($config->appsecret)
                  . ($config->aeskey ? '<br>EncodingAESKey: ' . xwechat_config_mask($config->aeskey) : '')
                  . ($config->bundle == 'wechatcorp' ? '<br>AgentID: ' . $config->agentid : ''),
      'bundle' => isset($bundles[$config->bundle]) ? $bundles[$config->bundle] : $config->bundle,
      'updated' => $config->updated ? date('Y-m-d H:i:s', $config->updated) : date('Y-m-d H:i:s', $config->created),
      'operate'   => array(
        'data' => array(
          '#theme' => 'links__node_operations',
          '#links' => $links,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      ),
    );
  }
  if (empty($rows)) {
    $rows[] = array(array(
        'data' => t('No xwechat config available.'),
        'colspan' => '7',
      ));
  }

  $header = array(t('WID'), t('Qrcode'), t('Name'), t('Config'), t('Bundle'), t('Updated'), t('Operation'));
  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'xwechat-configs')));

  $form = array();
  $form['list'] = array(
    '#type' => 'fieldset',
    '#title' => t('xwechat configs'),
    '#collapsible' => TRUE,
  );
  $form['list']['table'] = array(
    '#type' => 'item',
    '#prefix' => '<div>',
    '#markup' => $output,
    '#suffix' => '</div>',
  );

  // add xwechat config.
  $form['config'] = array(
    '#type' => 'fieldset',
    '#title' => t('Add xwechat config'),
  );
  $form['config']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Config name'),
    '#description' => t('A config with this same name will be created.'),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['config']['username'] = array(
    '#type' => 'textfield',
    '#title' => t('UserName'),
    '#default_value' => '',
    '#maxlength' => 128,
  );
  $form['config']['token'] = array(
    '#type' => 'textfield',
    '#title' => t('Token'),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['config']['appid'] = array(
    '#type' => 'textfield',
    '#title' => t('Appid'),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['config']['appsecret'] = array(
    '#type' => 'textfield',
    '#title' => t('Appsecret'),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['config']['aeskey'] = array(
    '#type' => 'textfield',
    '#title' => t('Aeskey'),
    '#default_value' => '',
  );
  $form['config']['bundle'] = array(
    '#type' => 'select',
    '#title' => t('Bundle'),
    '#default_value' => 'wechat',
    '#options' => array('wechat'=>'公众号', 'wechatcorp' => '企业号'),
  );
  $form['config']['agentid'] = array(
    '#type' => 'textfield',
    '#title' => t('AgentID'),
    '#default_value' => '0',
  );

  $form['config']['op'] = array(
    '#type' => 'submit',
    '#value' => t('Add config'),
  );
  
  return $form;
}

/**
 * Validate "Add config" form.
 */
function xwechat_config_add_form_validate($form, &$form_state) {
  $configs = xwechat_get_configs(NULL);
  $config_name = array();
  foreach ($configs as $config) {
    $config_name[] = $config->name;
  }

  if (!empty($config_name)) {
    // Check if name is unique
    if (in_array($form_state['values']['name'], $config_name)) {
      form_set_error('', t('Xwechat config %s already exists. Please use a different name.', array('%s' => $form_state['values']['name'])));
    }
  }
}
/**
 * Add xwechat config to database from "Add config" form.
 */
function xwechat_config_add_form_submit($form, &$form_state) {
  if(!empty($form_state['values'])){
    $wid = db_insert('xwechat_config')
    ->fields(array(
      'name' => $form_state['values']['name'],
      'username' => $form_state['values']['username'],
      'token' => $form_state['values']['token'],
      'appid' => $form_state['values']['appid'],
      'appsecret' => $form_state['values']['appsecret'],
      'aeskey' => $form_state['values']['aeskey'],
      'bundle' => $form_state['values']['bundle'],
      'agentid' => $form_state['values']['agentid'],
      'created' => time(),
    ))
    ->execute();
  }

  if ($wid) {
    drupal_set_message(t('XWechat config %s added.', array('%s' => $form_state['values']['name'])));
    try {
      $wechatconfig = xwechat_config_load($wid);
      $wechat = new WeChat($wechatconfig);
      $wechat->getAccessToken();
    } catch (Exception $e) {
      drupal_set_message(t('But the configuration did not pass validation'), 'warning');
    }
  } else {
    drupal_set_message(t('XWechat config saved failed.'), 'error');
  } 
}

/**
 * Edit config form.
 */
function xwechat_config_edit_form($form, &$form_state, $wid) {
  $config = xwechat_get_configs($wid);

  $form = array();

  $form['config']['wid'] = array(
    '#type' => 'hidden',
    '#value' => $wid,
  );

  $form['config']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Config name'),
    '#description' => t('The config name must be unique.'),
    '#default_value' => $config->name,
    '#required' => TRUE,
  );
  $form['config']['username'] = array(
    '#type' => 'textfield',
    '#title' => t('UserName'),
    '#default_value' => $config->username,
    '#maxlength' => 128,
  );
  $form['config']['token'] = array(
    '#type' => 'textfield',
    '#title' => t('Token'),
    '#default_value' => $config->token,
    '#required' => TRUE,
  );
  $form['config']['appid'] = array(
    '#type' => 'textfield',
    '#title' => t('Appid'),
    '#default_value' => $config->appid,
    '#required' => TRUE,
  );
  $form['config']['appsecret'] = array(
    '#type' => 'textfield',
    '#title' => t('Appsecret'),
    '#default_value' => $config->appsecret,
    '#required' => TRUE,
  );
  $form['config']['aeskey'] = array(
    '#type' => 'textfield',
    '#title' => t('Aeskey'),
    '#default_value' => $config->aeskey,
  );
  $form['config']['bundle'] = array(
    '#type' => 'select',
    '#title' => t('Bundle'),
    '#default_value' => $config->bundle,
    '#options' => array('wechat'=>'公众号', 'wechatcorp' => '企业号'),
  );
  $form['config']['agentid'] = array(
    '#type' => 'textfield',
    '#title' => t('AgentID'),
    '#default_value' => $config->agentid,
  );

  $form['op'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Validate edit config form.
 */
function xwechat_config_edit_form_validate($form, &$form_state) {
  $configs = xwechat_get_configs(NULL);
  $config_name = array();
  foreach ($configs as $config) {
    $config_name[$config->wid] = $config->name;
  }

  // Remove current config name to prevent false error.
  unset($config_name[$form_state['values']['wid']]);

  if (!empty($config_name)) {
    // Check if name is unique.
    if (in_array($form_state['values']['name'], $config_name)) {
      form_set_error('', t('XWechat config %s already exists. Please use a different name.', array('%s' => $form_state['values']['name'])));
    }
  }
}

/**
 * Submit edit config form.
 */
function xwechat_config_edit_form_submit($form, &$form_state) {
  $wid = db_update('xwechat_config')
    ->fields(array(
      'name' => $form_state['values']['name'],
      'username' => $form_state['values']['username'],
      'token' => $form_state['values']['token'],
      'appid' => $form_state['values']['appid'],
      'appsecret' => $form_state['values']['appsecret'],
      'aeskey' => $form_state['values']['aeskey'],
      'bundle' => $form_state['values']['bundle'],
      'agentid' => $form_state['values']['agentid'],
      'updated' => time(),
    ))
  ->condition('wid', $form_state['values']['wid'])
  ->execute();
  
  if ($wid) {
    drupal_set_message(t('XWechat config updated.'));
    try {
      $wechatconfig = xwechat_config_load($wid);
      $wechat = new WeChat($wechatconfig);
      $wechat->getAccessToken();
    } catch (Exception $e) {
      drupal_set_message(t('But the configuration did not pass validation'), 'warning');
    }
    $form_state['redirect'] = 'admin/structure/xwechat';
    return $form;
  } else {
    drupal_set_message(t('XWechat config saved failed.'), 'error');
  }
  
}

/**
 * Delete config form.
 */
function xwechat_config_confirm_delete_form($form, &$form_state, $wid) {
  $config = xwechat_get_configs($wid);
  if (empty($config)) {
    drupal_set_message(t('The config with wid @wid was not found.', array('@wid' => $wid)), 'error');
    return array();
  }
  else {
    $question = t('Are you sure you want to delete the xwechat config %name?', array('%name' => $config->name));
    // The page to go to if the user denies the action.
    $path = 'admin/structure/xwechat';

    // Additional text to display (defaults to "This action cannot be undone.").
    $description = t('This action cannot be undone.');

    // A caption for the button which confirms the action.
    $yes = t('Delete');

    // A caption for the link which denies the action.
    $no = t('Cancel');

    // set wid value to use in submit function.
    $form['wid'] = array(
      '#type' => 'value',
      '#value' => $wid,
    );

    // set the redirect path value to use in submit function.
    $form_state['redirect'] = $path;
    return confirm_form(
      $form,
      $question,
      $path,
      $description,
      $yes,
      $no
    );
  }
}

/**
 * Delete a xwechat config.
 */
function xwechat_config_confirm_delete_form_submit($form, &$form_state) {

  if (xwechat_config_confirm_delete($form_state['values']['wid'])) {
    $succes_message = t('XWechat config successfully deleted!');
    drupal_set_message($succes_message);
  }
  else {
    $failure_message = t('There was a problem deleting the xwechat config');
    drupal_set_message($failure_message);
  }
  $form_state['redirect'] = 'admin/structure/xwechat';
  return $form;
}

/**
 * Delete a xwechat config from the database.
 */
function xwechat_config_confirm_delete($wid) {
  $result = db_delete('xwechat_config')
  ->condition('wid', (int) $wid)
  ->execute();

  return TRUE;
}

/**
 * 显示星号.
 */
function xwechat_config_mask($string) {
  $len = strlen($string);
  if ($len < 4) { return $string; }
  $num = floor($len*2/5);
  $rep = str_repeat('*', $num);
  $srt = floor($len/3) - 1;
  return substr_replace($string, $rep, $srt, $num);
}

/**
 * Implements hook_xwechat_operation().
 */
function xwechat_xwechat_operation($links = array(), $config) {
  $links['edit'] = array(
    'title' => t('Edit'),
    'href' => 'admin/structure/xwechat/config/edit/' . $config->wid,
  );

  $links['delete'] = array(
    'title' => t('Delete'),
    'href' => 'admin/structure/xwechat/config/delete/' . $config->wid,
  );
    
  return $links;
}
